#include <stdio.h>

void quicksort();

main(argc, argv) 
   int argc;
   char *argv[];
{
   FILE *fd, *fdo;
   int framecount;
   double *vstream;
   int fnum, fsize;
   double rate, time;
   int i;
   double base;
   int count;
   int countflag = 0;

   if (argc < 3) {
      printf("Usage: pdf [infile] [outfile] [countflag]\n");
      exit(1);
   }
   
   fd = fopen(argv[1], "r");
   if (fd == 0) { 
      printf("Error opening file\n");
      exit(1);
   }

   fdo = fopen(argv[2], "w");
   if (fdo == 0) { 
      printf("Error opening file\n");
      exit(1);
   }

   if (argc > 3) {
      countflag = atoi(argv[3]); 
   }

   framecount = 0;
   while (!feof(fd)) {
      fscanf(fd,"%d %d %lf %lf\n", &fnum, &fsize, &rate, &time);
      framecount++;
   } 
   vstream = (double *) malloc((framecount+1)*sizeof(double));
   if (vstream == NULL) {
      printf("Malloc problem\n");
      exit(1);
   }

   /* 
    * We use vstream indexes 1..framecount to facilitate
    * quicksort algorithm.
    */
   fseek(fd,0,0);
   while (!feof(fd)) {
      fscanf(fd,"%d %d %lf %lf\n", &fnum, &fsize, &rate, &time);
      vstream[fnum+1] = rate/1000000;  /* Mbits/sec */ 
   } 

   quicksort(vstream, 1, framecount);  /* Sort the rates */
 
#define INTERVAL 0.05

   base = 0;
   count = 0;   
   fprintf(fdo,"%.4lf %d\n",0.0,0); 
   i = 1;
   while (i <= framecount) {
      if ((vstream[i] > base) && (vstream[i] <= base+INTERVAL)) {
         count++;
         i++;
      } else {
         if (countflag) {
            fprintf(fdo,"%.4lf %.4lf %d\n",base+INTERVAL,
                    (count*1.0)/framecount, count);
         } else {
            fprintf(fdo,"%.4lf %.4lf\n",base+INTERVAL,
                    (count*1.0)/framecount);
         }
         count = 0;   
         base += INTERVAL;
      }
   }
   if (countflag) {
      fprintf(fdo,"%.4lf %.4lf %d\n",base+INTERVAL, 
              (count*1.0)/framecount, count);
   } else {
      fprintf(fdo,"%.4lf %.4lf\n",base+INTERVAL, 
              (count*1.0)/framecount);
   }

   fclose(fd);
   fclose(fdo);
}

void quicksort(a, l, r)
   int r, l;
   double *a;
{
   int i,j;
   double v,t;
   
   if (r > l) {
      v = a[r];
      i = l - 1;
      j = r;
      do {
         do {
            i++;
         } while (a[i] < v);
         do {
            j--;
         } while (a[j] > v);
         t = a[i];
         a[i] = a[j];
         a[j] = t;
      } while (j > i);  
      a[j] = a[i];
      a[i] = a[r];
      a[r] = t;
      quicksort(a, l, i-1);
      quicksort(a, i+1, r);
   } 
} 



